-- **************************************************************************
--   TitanPalsConfig.lua
--
--   By: KanadiaN
--        (Lindarena@Laughing Skull)
-- **************************************************************************

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Titan", true );
local LB = LibStub( "AceLocale-3.0" ):GetLocale( "TitanPals", true );
local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
local AceConfig = LibStub( "AceConfig-3.0" );

-- **************************************************************************
-- NAME : Core Vars
-- DESC : These are here so they do not get changed like the locales would
-- **************************************************************************

palsDB = nil;
palsAuthor = nil;
TitanPalsFormatToRemove = "None";

local PlugInName = "TitanPals";
LB["TITAN_PALS_CORE_ID"] = "Pals";
LB["TITAN_PALS_CORE_VERSION"] = GetAddOnMetadata( PlugInName, "Version" );
LB["TITAN_PALS_CORE_DTAG"] = "|cffff0000<TitanPals Debug>|r %s";
LB["TITAN_PALS_CORE_ITAG"] = "|cffff9900<TitanPals>|r %s";
LB["TITAN_PALS_CORE_ETAG"] = "|cffff9900<TitanPals Error>|r %s";
LB["TITAN_PALS_CORE_ADDVER"] = "r68";
LB["TITAN_PALS_CORE_ARTWORK"] = "Interface\\FriendsFrame\\UI-Toast-ChatInviteIcon";
LB["TITAN_PALS_CONFIG_DEBUG"] = "Debug";
LB["TITAN_PALS_CONFIG_DEBUG_DESC"] = "Enable Debug Mode";
LB["TITAN_PALS_CONFIG_HEADER_DEBUG"] = "Debugging"

-- **************************************************************************
-- NAME : TitanPanelPals_ConfigUtils(a)
-- DESC : Pull information from .toc for config dialog
-- **************************************************************************
local function TitanPanelPals_ConfigUtils( a )
     if ( a == "GetAuthor" ) then
          return GetAddOnMetadata( PlugInName, "Authors" );
     elseif ( a == "GetTranslation" ) then
          return GetAddOnMetadata( PlugInName, "X-Translation" );
     elseif ( a == "GetCategory" ) then
          return GetAddOnMetadata( PlugInName, "X-Category" );
     elseif ( a == "GetEmail" ) then
          return GetAddOnMetadata( PlugInName, "X-Email" );
     elseif ( a == "GetWebsite" ) then
          return GetAddOnMetadata( PlugInName, "X-Website" );
     elseif ( a == "GetVersion" ) then
          return tostring( GetAddOnMetadata( PlugInName, "Version" ) );
     end
end

-- **************************************************************************
-- NAME : PalsOpt
-- DESC : This is the root of the config dialog
-- **************************************************************************
local PalsOpt = {
     type = "group",
     name = LB["TITAN_PALS_ADDON_LABEL"],
     args = {
          confgendesc = {
               type = "description", order = 1,
               name = LB["TITAN_PALS_ABOUT_DESC"],
               cmdHidden = true
          },
	  cat1 = {

               type = "group", order = 2, inline = true,
               name = LB["TITAN_PALS_ABOUT"],
               args = {
                    confversiondesc = {
                         type = "description", order = 1,
                         name = LB["TITAN_PALS_ABOUT_VERSION"]:format( TitanPanelPals_ConfigUtils( "GetVersion" ) ),
                         cmdHidden = true
                    },
                    confauthordesc = {
                         type = "description", order = 2,
                         name = LB["TITAN_PALS_ABOUT_AUTHOR"]:format( TitanPanelPals_ConfigUtils( "GetAuthor" ) ),
                         cmdHidden = true
                    },
                    conflangauthordesc = {
                         type = "description", order = 3,
                         name = LB["TITAN_PALS_ABOUT_TRANSLATION"]:format( TitanPanelPals_ConfigUtils( "GetTranslation" ) ),
                         cmdHidden = true
                    },
                    confcatdesc = {
                         type = "description", order = 4,
                         name = LB["TITAN_PALS_ABOUT_CATEGORY"]:format( TitanPanelPals_ConfigUtils( "GetCategory" ) ),
                         cmdHidden = true
                    },
                    confemaildesc = {
                         type = "description", order = 5,
                         name = LB["TITAN_PALS_ABOUT_EMAIL"]:format( TitanPanelPals_ConfigUtils( "GetEmail" ) ),
                         cmdHidden = true
                    },
                    confwebsitedesc = {
                         type = "description", order = 6,
                         name = LB["TITAN_PALS_ABOUT_WEBSITE"]:format( TitanPanelPals_ConfigUtils( "GetWebsite" ) ),
                         cmdHidden = true
                    },
                    confmemorydesc = {
                         type = "description", order = 7,
                         name = function()
                              UpdateAddOnMemoryUsage();
                              mem = floor( GetAddOnMemoryUsage( PlugInName ) );
                              return LB["TITAN_PALS_ABOUT_MEMORY"]:format( mem );
                         end,
                         cmdHidden = true,
                    },
               },
	  },
	  cat2 = {
	       type = "group", order = 3,
	       name = LB["TITAN_PALS_CONFIG_SET_HEADER_2"],
	       args = {
                    tooltipshowignored = {
                         type = "toggle", order = 1,
                         name = LB["TITAN_PALS_CONFIG_SHOWIGNORED"],
                         desc = LB["TITAN_PALS_CONFIG_SHOWIGNORED_DESC"],
                         get = function() return TitanPals.Settings.ShowIgnored; end,
                         set = function( _, value )
                              TitanPals.Settings.ShowIgnored = value;
                              TitanPanelButton_UpdateButton( LB["TITAN_PALS_CORE_ID"] );
                         end,
                    },
                    tooltipshowmem = {
                         type = "toggle", order = 2,
                         name = LB["TITAN_PALS_CONFIG_SHOWMEM"],
                         desc = LB["TITAN_PALS_CONFIG_SHOWMEM_DESC"],
                         get = function() return TitanPals.Settings.ShowMem; end,
                         set = function( _, value ) TitanPals.Settings.ShowMem = value; end,
                    },
                    tooltipbannor = {
                         type = "toggle", order = 2,
                         name = LB["TITAN_PALS_CONFIG_BANNOR"],
                         desc = LB["TITAN_PALS_CONFIG_BANNOR_DESC"],
                         get = function() return TitanPals.Settings.DisplayB; end,
                         set = function( _, value ) TitanPals.Settings.DisplayB = value; end,
                    },
		    tooltipshowoffline = {
                         type = "toggle", order = 4,
                         name = LB["TITAN_PALS_CONFIG_SHOWOFFLINE"],
                         desc = LB["TITAN_PALS_CONFIG_SHOWOFFLINE_DESC"],
                         get = function() return TitanPals.Settings.ShowOffline; end,
                         set = function( _, value ) TitanPals.Settings.ShowOffline = value; end,
                    },
                    debugmode = {
                         type = "toggle", order = 5,
                         name = LB["TITAN_PALS_CONFIG_DEBUG"],
                         desc = LB["TITAN_PALS_CONFIG_DEBUG_DESC"],
                         hidden = function() return not TitanPals_CoreUtils( "Authors", ( "%s@%s" ):format( TitanPals.Profile.Player, TitanPals.Profile.Realm ) ) end,
                         get = function() return TitanPals.Debug.State; end,
                         set = function( _, value ) TitanPals.Debug.State = value; end,
                    },
                    tooltipnoalts = {
                         type = "toggle", order = 6, width = "full",
                         name = LB["TITAN_PALS_CONFIG_NOALTS"],
                         desc = LB["TITAN_PALS_CONFIG_NOALTS_DESC"],
                         get = function() return TitanPals.Settings.NoAlts; end,
                         set = function( _, value ) TitanPals.Settings.NoAlts = value; end,
                    },
               },
	  },
	  cat3 = {
               type = "group", order = 4,
               name = LB["TITAN_PALS_CONFIG_SET_HEADER"],
               args = {
	            tooltippresets = {
                         type = "select", order = 1, width = "full",
                         name = LB["TITAN_PALS_CONFIG_PRESETS"],
                         desc = LB["TITAN_PALS_CONFIG_PRESETS_DESC"],
                         get = function() return TitanPals.Settings.TFormat; end,
                         set = function( _, value ) TitanPals.Settings.TFormat = value; end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPals.presets ) do
                                   if ( v.value ~= TitanPals.Settings.TFormat ) then
                                        preList[v.value] = ( "|cff19ff19%s|r" ):format( v.value );
                                   else
                                        preList[v.value] = ( "|cffffff9a%s|r" ):format( v.value );
                                   end
                              end
                              table.sort( preList, function( a, b )
                                   return string.lower( TitanPals.persets[a].name ) < string.lower( TitanPals.persets[b].name );
                              end );
                              return preList;
                         end
                    },
                    nulloption1 = {
                         type = "description", order = 2,
                         name = LB["TITAN_PALS_CONFIG_PRESETS_PREVIEW"],
                         cmdHidden = true,
                    },
               },
	  },
	  cat4 = {
               type = "group", order = 5,
               name = LB["TITAN_PALS_CONFIG_HEADER_ADD"],
               args = {
                    tooltipaddcustom = {
			 type = "input", order = 1, width = "full",
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP"],
			 desc = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_DESC"],
			 get = function() return end,
			 set = function( _, value )
                              local tc = table.getn( TitanPals.presets );
			      tc = ( tc + 1 );
			      local tName = ( "Style #%s" ):format( tc );
			      local tValue = value;
			      TitanPanelPals_AddNewPreset( tName, tValue );
			 end,
		    },
                    nulloptionct = {
			 type = "description", order = 2,
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_INST"],
			 cmdHidden = true;
                    },
               },
	  },
          cat5 = {
               type = "group", order = 6,
               name = LB["TITAN_PALS_CONFIG_HEADER_REMOVE"],
	       args = {
	            tooltippresetsremove = {
                         order = 1, type = "select", width = "full",
                         name = LB["TITAN_PALS_CONFIG_PRESETS"],
                         desc = LB["TITAN_PALS_CONFIG_REMOVE_PRESETS_DESC"],
                         get = function() return TitanPalsFormatToRemove; end,
                         set = function( _, value ) 
                              TitanPals.Temp["TitanPalsFormatToRemove"] = value;
                              TitanPalsFormatToRemove = value; 
			 end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPals["presets"] ) do
                                   if ( v.value ~= TitanPals.Settings.TFormat ) then
                                        preList[v.value] = ( "|cff19ff19%s %s|r" ):format( v.name, v.value );
                                   else
                                        preList[v.value] = ( "|cffffff9a%s %s|r" ):format( v.name, v.value );
                                   end
                              end
                              if ( TitanPalsFormatToRemove ~= "None" ) then
                                   preList["None"] = ( "|cff19ff19%s|r" ):format( L["TITAN_NONE"] );
                              else
                                   preList["None"] = ( "|cffffff9a%s|r" ):format( L["TITAN_NONE"] );
                              end
                              return preList;
                         end
                    },
		    tooltippresetremovebutton = {
                         type = "execute", order = 2,
			 name = LB["TITAN_PALS_CONFIG_REMOVE"],
			 desc = LB["TITAN_PALS_CONFIG_REMOVE_DESC"],
                         func = function()
                              if ( TitanPalsFormatToRemove == "None" ) then return; end
                              -- Need to add a check to see if the TFormat is about to be removed and if so change the TFormat to default
                              local k, v, z;
                              for k, v in pairs( TitanPals.presets ) do
                                   if ( TitanPals.Settings.TFormat == TitanPalsFormatToRemove and not z ) then 
                                        TitanPals.Settings.TFormat = v.value;
                                        z = true;
                                   end
                                   if ( v.value == TitanPalsFormatToRemove ) then
                                        table.remove( TitanPals.presets, k );
                                        TitanPalsFormatToRemove = "None";
                                        break;
                                   end
                              end
                         end,
		    },
               },
          },
	  cat6 = {
	       type = "group", order = 6,
               name = LB["TITAN_PALS_CONFIG_SET_HEADER_STATUS"],
               args = {
                    statustypes = {
                         type = "select", order = 1, width = "full",
                         name = LB["TITAN_PALS_CONFIG_STATUS_TYPES"],
                         desc = LB["TITAN_PALS_CONFIG_STATUS_TYPES_DESC"],
                         get = function() return TitanPals.Settings.SFormat; end,
                         set = function( _, value ) TitanPals.Settings.SFormat = value; end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPals.sTypes ) do
                                   if ( v.value ~= TitanPals.Settings.SFormat ) then
                                        preList[v.value] = ( "|cff19ff19%s|r" ):format(v.value );
                                   else
                                        preList[v.value] = ( "|cffffff9a%s|r" ):format( v.value );
                                   end
                              end
                              return preList;
                         end
                    },
                    nulloptionct = {
                         type = "description", order = 2,
                         name = LB["TITAN_PALS_CONFIG_STATUS_TYPES_INST"],
                         cmdHidden = true
                    },
               },
          },
          cat7 = {
	       type = "group", order = 7,
	       name =  LB["TITAN_PALS_CONFIG_HEADER_REALID"];
	       args = {
                    realidonoroff = {
			 name = LB["TITAN_PALS_CONFIG_REALID_ONOFF"],
			 desc = LB["TITAN_PALS_CONFIG_REALID_ONOFF_DESC"],
                         type = "toggle", order = 1, width = "full",
			 get = function() return TitanPals.Settings.ShowBNTooltip; end,
			 set = function( _, value ) 
                              TitanPals.Settings.ShowBNTooltip = value;
                              TitanPanelButton_UpdateButton( LB["TITAN_PALS_CORE_ID"] );
                         end,
		    },
		    tooltipacustomrealid = {
			 type = "select", order = 2, width = "full",
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_REALID"],
			 desc = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_REALID_DESC"],
			 get = function() return TitanPals.Settings.RealIDFormat end,
			 set = function( _, value ) TitanPals.Settings.RealIDFormat = value; end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPals.realidformats ) do
                                   if ( v.value ~= TitanPals.Settings.RealIDFormat ) then
                                        preList[v.value] = ( "|cff19ff19%s|r" ):format( v.value );
                                   else
                                        preList[v.value] = ( "|cffffff9a%s|r" ):format( v.value );
                                   end
                              end
                              return preList;
                         end
		    },
                    nulloption1 = {
                         type = "description", order = 3,
                         name = "\n",
                         cmdHidden = true,
                    },
                    confrealinondesc = {
                         type = "description", order = 4,
                         name = LB["TITAN_PALS_CONFIG_REALID_NOTE1_ON"],
                         cmdHidden = true,
                    },
                    nulloption2 = {
                         type = "description", order = 5,
                         name = "\n",
                         cmdHidden = true,
                    },
                    confrealidoffdesc = {
                         type = "description", order = 6,
                         name = LB["TITAN_PALS_CONFIG_REALID_NOTE1_OFF"],
                         cmdHidden = true,
                    },
               },
	  },
	  cat8 = {
               type = "group", order = 8,
               name = LB["TITAN_PALS_CONFIG_HEADER_REALID_ADD"],
               args = {
                    tooltipaddrealidcustom = {
			 type = "input", order = 1, width = "full",
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_REALID_TOOLTIP"],
			 desc = LB["TITAN_PALS_CONFIG_CUSTOM_REALID_TOOLTIP_DESC"],
			 get = function() return end,
			 set = function( _, value )
                              local tc = table.getn( TitanPals.realidformats );
			      tc = ( tc + 1 );
			      local tName = ( "Format #%s" ):format( tc );
			      local tValue = value;
			      TitanPanelPals_AddNewRealIDPreset( tName, tValue );
			 end,
		    },
                    nulloptionct = {
			 type = "description", order = 2,
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_REALID_TOOLTIP_INST"],
			 cmdHidden = true,
                    },
               },
	  },
          cat9 = {
               type = "group", order = 9,
               name = LB["TITAN_PALS_CONFIG_HEADER_REALID_REMOVE"],
	       args = {
	            tooltippresetsremove = {
                         type = "select", order = 1, width = "full",
                         name = LB["TITAN_PALS_CONFIG_REALID_PRESETS"],
                         desc = LB["TITAN_PALS_CONFIG_REALID_REMOVE_PRESETS_DESC"],
                         get = function() return TitanPalsFormatToRemove; end,
                         set = function( _, value ) 
                              TitanPals.Temp["TitanPalsFormatToRemove"] = value;
                              TitanPalsFormatToRemove = value;
			 end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPals.realidformats ) do
                                   if ( v.value ~= TitanPals.Settings.RealIDFormat ) then
                                        preList[v.value] = ( "|cff19ff19%s %s|r" ):format( v.name, v.value );
                                   else
                                        preList[v.value] = ( "|cffffff9a%s %s|r" ):format( v.name, v.value );
                                   end
                              end
                              if ( TitanPalsFormatToRemove ~= "None" ) then
                                   preList["None"] = ( "|cff19ff19%s|r" ):format( L["TITAN_NONE"] );
                              else
                                   preList["None"] = ( "|cffffff9a%s|r" ):format( L["TITAN_NONE"] );
                              end
                              return preList;
                         end
                    },
		    tooltippresetremovebutton = {
                         type = "execute", order = 2,
			 name = LB["TITAN_PALS_CONFIG_REALID_REMOVE"],
			 desc = LB["TITAN_PALS_CONFIG_REALID_REMOVE_DESC"],
                         func = function()
                              if ( TitanPalsFormatToRemove == "None" ) then return; end
                              -- Need to add a check to see if the RealIDFormat is about to be removed and if so change the RealIDFormat to default
                              local k, v, z;
                              for k, v in pairs( TitanPals.realidformats ) do
                                   if ( TitanPals.Settings.RealIDFormat == TitanPalsFormatToRemove and not z ) then 
                                        TitanPals.Settings.RealIDFormat = v.value;
                                        z = true;
                                   end
                                   if ( v.value == TitanPalsFormatToRemove ) then
                                        table.remove( TitanPals.realidformats, k );
                                        TitanPalsFormatToRemove = "None";
                                        break;
                                   end
                              end
                         end,
		    },
               },
          },
          cat10 = {
               type = "group", order = 10,
               name = LB["TITAN_PALS_CONFIG_HEADER_DEBUG"],
               hidden = function() return not TitanPals.Debug.State; end,
               args = {
                    
               },
          },
     },
}

-- **************************************************************************
-- NAME : TitanPanelPals_AddNewPreset()
-- DESC : This will add a custom tooltip preset
-- **************************************************************************
function TitanPanelPals_AddNewPreset( presetname, presetvalue )
     local found
     for _,i in pairs( TitanPals.presets ) do
          if i.name == presetname or i.value == presetvalue then
               found = true; break;
          end
     end	
     if ( not found ) then table.insert( TitanPals.presets, { name = presetname, value = presetvalue } ); end
end

-- **************************************************************************
-- NAME : TitanPanelPals_AddNewRealIDPreset()
-- DESC : This will add a custom tooltip preset for RealID Display
-- **************************************************************************
function TitanPanelPals_AddNewRealIDPreset( presetname, presetvalue )
     local found
     for _, i in pairs( TitanPals.realidformats ) do
          if ( i.name == presetname or i.value == presetvalue ) then
               found = true;
               break;
          end
     end	
     if ( not found ) then table.insert( TitanPals.realidformats, { name = presetname, value = presetvalue } ); end
end

-- **************************************************************************
-- NAME : TitanPanelPalsButton_OnLoad()
-- DESC : Registers the plugin upon it loading
-- **************************************************************************
function TitanPanelPalsButton_OnLoad( self )
     self.registry = {
          id = LB["TITAN_PALS_CORE_ID"],
          builtIn = false,
          version = TITAN_VERSION,
          menuText = LB["TITAN_PALS_MENU_TEXT"],
          buttonTextFunction = "TitanPanelPalsButton_GetButtonText";
	  tooltipTitle = LB["TITAN_PALS_TOOLTIP"],
          tooltipCustomFunction = TitanPanelPals_SetTooltip;
          icon = LB["TITAN_PALS_CORE_ARTWORK"],
          iconWidth = 16,
          category = "Information",
          savedVariables = {
	       ShowIcon = 1,
	       ShowLabelText = 1,
          }
     };
     self:RegisterEvent( "ADDON_LOADED" );
     self:RegisterEvent( "PLAYER_ENTERING_WORLD" );
     self:RegisterEvent( "BN_FRIEND_ACCOUNT_ONLINE" );
     self:RegisterEvent( "BN_FRIEND_ACCOUNT_OFFLINE" );
     self:RegisterEvent( "FRIENDLIST_UPDATE" );
     self:RegisterEvent( "IGNORELIST_UPDATE" );
     -- Titan Pals Config
     AceConfig:RegisterOptionsTable( PlugInName, PalsOpt );
     AceConfigDialog:AddToBlizOptions( PlugInName, LB["TITAN_PALS_CONFIG"] );
     TitanPals_Init();
end